use std::{collections::HashMap, sync::Arc};

use chrono::Utc;
use futures_util::lock::Mutex;
use jsonwebtoken::{decode, Algorithm, DecodingKey, Validation};
use lazy_static::lazy_static;
use oauth2::basic::BasicClient;
use oauth2::{reqwest, RevocationUrl};
use oauth2::{
    AuthUrl, AuthorizationCode, ClientId, ClientSecret, CsrfToken, PkceCodeChallenge, RedirectUrl,
    Scope, TokenResponse, TokenUrl,
};
use reqwest::Client;
use serde::Deserialize;
use serde_json::Value;

use crate::config;
use crate::model::response::ResErr;

use super::SocialResult;

lazy_static! {
    pub static ref GOOGLE_SOCIAL: GoogleSocial = GoogleSocial::default();
}

#[derive(Default)]
pub struct GoogleSocial {
    google_public_keys: Arc<Mutex<GooglePublicKeys>>,
}

/// 假设GOOGLE_PUBLIC_CERT_URL是Google提供的公钥URL
const GOOGLE_PUBLIC_CERT_URL: &str = "https://www.googleapis.com/oauth2/v3/certs";

/// 用途：授权端点
/// - 用户登录和授权的入口
/// - 获取用户同意访问特定范围的权限
const GOOGLE_AUTH_URL: &str = "https://accounts.google.com/o/oauth2/v2/auth";
/// 用途：令牌端点
/// - 使用授权码交换访问令牌
/// - 刷新访问令牌
/// - 获取新的访问令牌
const GOOGLE_TOKEN_URL: &str = "https://oauth2.googleapis.com/token";
/// 用途：令牌撤销端点
/// - 撤销访问令牌
/// - 撤销刷新令牌
/// - 用户退出登录时使用
const GOOGLE_REVOCATION_URL: &str = "https://oauth2.googleapis.com/revoke";
/// 用途：Google Play Games 权限范围
/// - 访问游戏相关API的权限
/// - 获取玩家信息、成就等
const GOOGLE_GAMES_SCOPE: &str = "https://www.googleapis.com/auth/games";
/// 用途：用户资料权限范围
/// - 访问用户基本资料的权限
/// - 获取用户名、头像等信息  
const GOOGLE_PROFILE_SCOPE: &str = "https://www.googleapis.com/auth/userinfo.profile";
/// 用途：获取玩家状态信息
/// 返回的信息包括：
/// - 游戏进度
/// - 玩家统计数据
/// - 游戏内状态
/// - 成就完成情况
/// - 排行榜数据等
/// 示例响应：
/// ```
/// {
///     "kind": "games#playerState",
///     "timeMillis": "1234567890",
///     "key": "gameState",
///     "data": {
///         // 游戏相关的状态数据
///         "progress": "...",
///         "achievements": [...],
///         "scores": [...],
///         // 其他游戏特定数据
///     }
/// }
/// ```
const PLAY_GAMES_PLAYER_STATS_URL: &str = "https://games.googleapis.com/games/v1/players/me/playerStates";
/// 用途：获取玩家基本资料
/// 返回的信息包括：
// - 玩家ID (playerId)
/// - 显示名称 (displayName)
/// - 头像URL (avatarImageUrl)
/// - 玩家等级
/// - 最后登录时间
/// - 玩家标签
/// 示例响应：
/// ```
/// {
///     "kind": "games#player",
///     "playerId": "123456789",
///     "displayName": "Player Name",
///     "avatarImageUrl": "https://...",
///     "bannerUrlLandscape": "https://...",
///     "bannerUrlPortrait": "https://...",
///     "profileSettings": {
///         "friendsListVisibility": "ALL"
///     },
///     "title": "Level 10 Player",
///     "lastPlayedWith": {
///         "kind": "games#played",
///         "timeMillis": "1234567890"
///     }
/// }
/// ```
const PLAY_GAMES_PROFILE_URL: &str = "https://games.googleapis.com/games/v1/players/me";

#[derive(Debug, Default, Deserialize)]
pub struct GoogleJwtProfile {
    // iss (issuer)：签发人
    pub iss: String,
    // sub (subject)：主题
    pub sub: String,
    pub azp: String,
    // aud (audience)：受众
    pub aud: String,
    // iat (Issued At)：签发时间
    pub iat: i64,
    // exp (expiration time)：过期时间
    pub exp: i64,
    pub email: String,
    pub email_verified: bool,
    pub at_hash: String,
    pub name: String,
    pub picture: String,
    pub given_name: String,
    pub family_name: String,
    pub locale: String,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub google_play_games_id: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub google_play_games_profile: Option<String>,
}

impl GoogleJwtProfile {
    #[allow(unused)]
    fn new() -> Self {
        GoogleJwtProfile {
            ..Default::default()
        }
    }
}

impl From<Value> for GoogleJwtProfile {
    fn from(value: Value) -> Self {
        let google_jwt_profile: GoogleJwtProfile =
            serde_json::from_value(value).expect("解析GoogleJwtProfile json失败");
        google_jwt_profile
    }
}

#[derive(Deserialize, Debug, Clone, Default)]
struct GooglePublicKey {
    e: String,
    n: String,
    kty: String,
    #[serde(rename = "use")]
    #[allow(unused)]
    use_: String,
    #[allow(unused)]
    alg: String,
    kid: String,
}

#[derive(Deserialize, Debug, Clone, Default)]
struct GooglePublicKeys {
    keys: Vec<GooglePublicKey>,
    #[serde(skip)]
    refresh_at: i64,
}

impl GoogleSocial {
    /// 异步获取并解析Google公钥
    async fn fetch_and_parse_google_public_keys(
        &self,
    ) -> SocialResult<HashMap<String, GooglePublicKey>> {
        let mut public_keys = self.google_public_keys.lock().await;
        if public_keys.keys.is_empty() || public_keys.refresh_at < (Utc::now().timestamp() - 30) {
            let response = Client::new().get(GOOGLE_PUBLIC_CERT_URL).send().await?;
            let mut google_keys: GooglePublicKeys = response.json().await?;
            tracing::info!("Google公钥获取成功, {:?}", google_keys);
            google_keys.refresh_at = Utc::now().timestamp() + 3600;
            *public_keys = google_keys;
        }

        let mut key_map = HashMap::new();
        // 解析公钥
        for key in public_keys.keys.iter() {
            if key.kty == "RSA" {
                key_map.insert(key.kid.to_owned(), key.to_owned());
            }
        }

        tracing::info!("Google公钥解析成功, {:?}", key_map);
        Ok(key_map)
    }

    /// 验证ID Token并考虑kid匹配
    pub async fn verify_id_token(&self, id_token: &str) -> SocialResult<GoogleJwtProfile> {
        // 获取并解析公钥
        let public_keys = self.fetch_and_parse_google_public_keys().await?;

        // 解码Token头部以获取kid
        let token_header = jsonwebtoken::decode_header(id_token).unwrap();
        let kid = token_header.kid;

        // 检查是否找到了有效的kid
        if kid.is_none() {
            return Err(Box::new(ResErr::social("校验Token失败,未找到有效的kid")));
        }
        let kid = kid.unwrap();

        // 根据kid找到正确的公钥
        let key = public_keys.get(&kid);

        match key {
            Some(key) => {
                // 验证Token
                let mut validation: Validation = Validation::new(Algorithm::RS256);
                validation.set_issuer(&["https://accounts.google.com", "accounts.google.com"]); // 设置预期的发行者
                validation.validate_aud = false;

                let decoded = decode::<Value>(
                    id_token,
                    &DecodingKey::from_rsa_components(key.n.as_str(), key.e.as_str()).unwrap(),
                    &validation,
                )?;

                let claims: Value = decoded.claims;
                let google_jwt_profile = GoogleJwtProfile::from(claims);

                // 校验有效期
                if google_jwt_profile.exp < Utc::now().timestamp() {
                    return Err(Box::new(ResErr::social("校验Token失败,token有效期无效")));
                }

                Ok(google_jwt_profile)
            }
            None => {
                self.verify_play_games_token(id_token).await
            }
        }
    }

    /// 验证 Google Play Games 的身份信息
    /// 使用 OAuth2 token exchange 获取有效的 access token
    /// oauth2开发库参考https://github.com/ramosbugs/oauth2-rs/blob/main/examples/google.rs
    /// google game account账户校验参考：https://github.com/heroiclabs/nakama/blob/master/social/social.go
    /// TODO: redirect_uri 需要改为实际的回调地址
    pub async fn verify_play_games_token(
        &self,
        play_games_token: &str,
    ) -> SocialResult<GoogleJwtProfile> {
        // 1. 创建 OAuth2 client
        let client = BasicClient::new(ClientId::new(config!().social.google.client_id.clone()))
            .set_client_secret(ClientSecret::new(config!().social.google.client_secret.clone()))
            .set_auth_uri(AuthUrl::new(GOOGLE_AUTH_URL.to_string())?)
            .set_token_uri(TokenUrl::new(GOOGLE_TOKEN_URL.to_string())?)
            // Google supports OAuth 2.0 Token Revocation (RFC-7009)
            .set_revocation_url(
                RevocationUrl::new(GOOGLE_REVOCATION_URL.to_string())
                    .expect("Invalid revocation endpoint URL"),
            );

        // 2. 执行 token exchange
        let token_response = client
            .exchange_code(AuthorizationCode::new(play_games_token.to_string()))
            .request_async(&reqwest::Client::new())
            .await?;

        let access_token = token_response.access_token();

        // 3. 获取用户信息和额外信息
        let mut profile = self.fetch_play_games_profile(access_token.secret()).await?;
        self.enrich_profile_with_additional_info(&mut profile, access_token.secret()).await?;

        Ok(profile)
    }

    /// 使用额外信息丰富用户资料
    async fn enrich_profile_with_additional_info(
        &self,
        profile: &mut GoogleJwtProfile,
        access_token: &str,
    ) -> SocialResult<()> {
        if let Some(additional_info) = self.fetch_additional_player_info(access_token).await? {
            profile.google_play_games_profile = Some(serde_json::to_string(&additional_info)?);
        }
        Ok(())
    }

    /// 获取 Google Play Games 用户资料
    async fn fetch_play_games_profile(&self, access_token: &str) -> SocialResult<GoogleJwtProfile> {
        let client = Client::new();
        let response = client
            .get(PLAY_GAMES_PROFILE_URL)
            .header("Authorization", format!("Bearer {}", access_token))
            .send()
            .await?;

        if !response.status().is_success() {
            return Err(Box::new(ResErr::social(
                "获取 Google Play Games 用户资料失败",
            )));
        }

        let player_info: Value = response.json().await?;

        // 构建用户资料
        let mut profile = GoogleJwtProfile::default();

        // 设置基本信息
        if let Some(player_id) = player_info.get("playerId").and_then(|v| v.as_str()) {
            profile.sub = player_id.to_string();
            profile.google_play_games_id = Some(player_id.to_string());
        }

        // 设置名称
        if let Some(display_name) = player_info.get("displayName").and_then(|v| v.as_str()) {
            profile.name = display_name.to_string();
        }

        // 设置头像
        if let Some(avatar_image_url) = player_info.get("avatarImageUrl").and_then(|v| v.as_str()) {
            profile.picture = avatar_image_url.to_string();
        }

        Ok(profile)
    }

    /// 获取额外的玩家信息
    async fn fetch_additional_player_info(
        &self,
        access_token: &str,
    ) -> SocialResult<Option<Value>> {
        let client = Client::new();
        let response = client
            .get(PLAY_GAMES_PLAYER_STATS_URL)
            .header("Authorization", format!("Bearer {}", access_token))
            .send()
            .await?;

        if !response.status().is_success() {
            tracing::warn!("获取额外玩家信息失败: {}", response.status());
            return Ok(None);
        }

        let stats: Value = response.json().await?;
        Ok(Some(stats))
    }
}
