use std::{future::Future, pin::Pin, sync::Arc, time::Duration};

use chrono_tz::Tz;
use tokio_cron_scheduler::{Job, JobScheduler, JobSchedulerError};

pub use inventory::submit;

type TaskFun = dyn Fn() -> Pin<Box<dyn Future<Output = ()> + Send>> + Send + Sync;

pub trait TaskMethod: Send + Sync + 'static {
    fn ge_task(&self) -> Task;
}

pub struct Task {
    pub job: Arc<TaskFun>,
    pub cron: Option<String>,
    pub interval: Option<u64>,
    pub time_zone: Option<String>,
}

/// 启动定时任务
pub async fn start() {
    let tasks = get_task_methods();
    match schedule(tasks).await {
        Ok(_) => tracing::info!("定时任务启动成功"),
        Err(err) => tracing::error!(error = ?err, "定时任务启动失败"),
    }
}

async fn schedule(tasks: Vec<Arc<Task>>) -> Result<(), JobSchedulerError> {
    let mut scheduler = JobScheduler::new().await?;
    scheduler.init().await?;

    // 添加任务
    for task in tasks {
        if task.cron.is_some() {
            let schedule = task.cron.as_ref().unwrap().as_str();
            if let Some(time_zone) = task.time_zone.as_ref() {
                let tz: Tz = time_zone.parse().unwrap();
                let job_fun = task.job.clone();
                let _task_uuid = scheduler
                    .add(Job::new_cron_job_async_tz(
                        schedule,
                        tz,
                        move |_uuid, _l| job_fun(),
                    )?)
                    .await?;
            } else {
                let job_fun = task.job.clone();
                let _task_uuid = scheduler
                    .add(Job::new_cron_job_async(schedule, move |_uuid, _l| {
                        job_fun()
                    })?)
                    .await?;
            }
        } else if task.interval.is_some() {
            let schedule = task.interval.unwrap();
            let job_fun = task.job.clone();
            let _task_uuid = scheduler
                .add(Job::new_repeated_async(
                    Duration::from_secs(schedule),
                    move |_uuid, _l| job_fun(),
                )?)
                .await?;
        }
    }

    // 添加关闭监听
    scheduler.set_shutdown_handler(Box::new(|| {
        Box::pin(async move {
            tracing::info!("定时任务已关闭");
        })
    }));

    // 启动任务
    scheduler.start().await?;
    Ok(())
}

inventory::collect!(&'static dyn TaskMethod);

#[macro_export]
macro_rules! submit_task {
    ($ty:ident) => {
        ::library::task::submit! {
            &$ty as &dyn ::library::task::TaskMethod
        }
    };
}

fn get_task_methods() -> Vec<Arc<Task>> {
    let mut result = Vec::<Arc<Task>>::new();
    for method in inventory::iter::<&dyn TaskMethod> {
        result.push(Arc::new(method.ge_task()));
    }
    result
}
